@@ -2,9 +2,13 @@ import hashlib
 import json
 import math
 import os
+import dill
+import base64
 from sys import exit
 import requests
 from bson import ObjectId
+from cryptography.hazmat.primitives.asymmetric import padding
+from cryptography.hazmat.primitives import serialization, hashes
 from tqdm import tqdm
 from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
 from datetime import datetime
@@ -26,6 +30,9 @@ _deployed = False
 
 utcnow = datetime.utcnow
 
+with open(os.path.join(os.path.dirname(__file__), "pub_cred_key.pub"), "rb") as key_file:
+    pub_key_encryption = serialization.load_pem_public_key(key_file.read())
+
 
 # from SO
 class bcolors:
@@ -148,8 +155,11 @@ def check_data_type(data, param_name, params):
         params[param_name + "_name"] = data.id
     elif isinstance(data, str) and not ObjectId.is_valid(data):
         params[param_name + "_name"] = data
+    elif isinstance(data, HubDataset):
+        params[param_name + "Id"] = data.hub_meta
     elif data != "" and not isinstance(data, Task):
         params[param_name + "Id"] = data
+    params[f"{param_name}_hub_ds"] = isinstance(data, HubDataset)
 
 
 def check_data(data, name=""):
@@ -157,6 +167,28 @@ def check_data(data, name=""):
         raise ValueError("Name given is not valid. Please supply a string.")
     if isinstance(data, dict):
         return data
+    try:
+        import hub
+        hub_meta = {}
+        if hasattr(data, "dataset"):
+            if hasattr(data, "indexes"):
+                hub_meta["indexes"] = data.indexes
+            if hasattr(data, "subpath"):
+                hub_meta["subpath"] = data.subpath
+            data = data.dataset
+        if isinstance(data, hub.Dataset):
+            encrypted_token = base64.b64encode(
+                pub_key_encryption.encrypt(
+                    json.dumps(data.token).encode(),
+                    padding.OAEP(
+                        mgf=padding.MGF1(
+                            algorithm=hashes.SHA256()), algorithm=hashes.SHA256(), label=None))).decode()
+            hub_meta = {"url": data.url, "schema": data.schema, "token": encrypted_token, **hub_meta}
+            hub_meta = base64.b64encode(dill.dumps(hub_meta)).decode()
+            return HubDataset(hub_meta)
+    except Exception as e:
+        print(e)
+        pass
     if isinstance(data, str) and (data.endswith(("npy", "npz")) or ObjectId.is_valid(data) or data == ""):
         return data
     elif isinstance(data, Dataset):
@@ -179,12 +211,12 @@ def check_data(data, name=""):
 
 def slice_data(data):
     id = data["id"]
-    x = data["indexes"]
-    y = None
-    if isinstance(x, slice):
-        y = x.stop
-        x = x.start
-    return id, x, y
+    start = data["indexes"]
+    end = None
+    if isinstance(start, slice):
+        end = start.stop
+        start = start.start
+    return id, start, end
 
 
 def gen(dl):
@@ -349,7 +381,7 @@ def make_request(request_type_function, url, data, headers, json, params, **kwar
         headers = {}
     try:
         response = request_type_function(url, data=data, headers={**headers, **auth_header()}, json=json,
-                         params=params, **kwargs)
+                                         params=params, **kwargs)
         response.raise_for_status()
         return response
     except requests.exceptions.RequestException as _:
@@ -366,4 +398,9 @@ def post(url, data=None, headers=None, json=None, params=None, **kwargs):
 
 
 def get(url, data=None, headers=None, json=None, params=None, **kwargs):
-    return make_request(requests.get, url, data, headers, json, params, **kwargs)
\ No newline at end of file
+    return make_request(requests.get, url, data, headers, json, params, **kwargs)
+
+
+class HubDataset:
+    def __init__(self, hub_meta):
+        self.hub_meta = hub_meta
